#version 330
uniform sampler2D colorTexture;
uniform vec2 pixelSize; // 1.0/width 1.0/height
uniform vec2 direction=vec2(1.0,0.0);
in vec2 fUVs;

float weights[5];
vec2 uv0[5];


out vec4 fragColor;


vec3 GuassianBlur()
{
	vec3 result = vec3(0.0);

	for(int i = 0; i < 5; i++)
	{
		result += texture2D(colorTexture,uv0[i]).rgb * weights[i];
	}

	return result;
}

void CalculateUvs_Weights()
{
	uv0[0] = fUVs + vec2(-pixelSize.x*2*direction.x,-pixelSize.y*2*direction.y);
	uv0[1] = fUVs + vec2(-pixelSize.x*direction.x,-pixelSize.y*direction.y);
	uv0[2] = fUVs + vec2(0,0);
	uv0[3] = fUVs + vec2(pixelSize.x*direction.x,pixelSize.y*direction.y);
	uv0[4] = fUVs + vec2(pixelSize.x*2*direction.x,pixelSize.y*2*direction.y);

	// Normalized weights
	weights[0] = 0.01330373 / 0.47365426;
	weights[1] = 0.11098164 / 0.47365426;
	weights[2] = 0.22508352 / 0.47365426;
	weights[3] = 0.11098164 / 0.47365426;
	weights[4] = 0.01330373 / 0.47365426;
}

void main()
{
	CalculateUvs_Weights();
	fragColor.rgb = GuassianBlur();
}
/**
// OLDVERSION //
#version 330

uniform sampler2D colorTexture;
uniform vec2 pixelSize; // 1.0/width 1.0/height
uniform float blurAmnt=0.00000009; // 1.0/width 1.0/height
uniform vec2 direction=vec2(1.0f,0.0f);
in vec2 fUVs;
out vec4 fragColor;
void main()
{
	  vec4 color;
 
    color += 1.0 * texture(colorTexture, fUVs + vec2(blurAmnt * -4.0*direction.x,blurAmnt * -4.0*direction.y));
    color += 2.0 * texture(colorTexture, fUVs + vec2(blurAmnt * -3.0*direction.x,blurAmnt * -3.0*direction.y));
    color += 3.0 * texture(colorTexture, fUVs + vec2(blurAmnt * -2.0*direction.x,blurAmnt * -2.0*direction.y));
    color += 4.0 * texture(colorTexture, fUVs + vec2(blurAmnt * -1.0*direction.x,blurAmnt * -1.0*direction.y));
 
    color += 5.0 * texture(colorTexture, fUVs + vec2(blurAmnt, 0));
 
    color += 4.0 * texture(colorTexture, fUVs + vec2(blurAmnt * 1.0*direction.x,blurAmnt * 1.0*direction.y));
    color += 3.0 * texture(colorTexture, fUVs + vec2(blurAmnt * 2.0*direction.x,blurAmnt * 2.0*direction.y));
    color += 2.0 * texture(colorTexture, fUVs + vec2(blurAmnt * 3.0*direction.x,blurAmnt * 3.0*direction.y));
    color += 1.0 * texture(colorTexture, fUVs + vec2(blurAmnt * 4.0*direction.x,blurAmnt * 4.0*direction.y));
 
    color /= 25.0;
	fragColor=vec4(color.rgb,1.0);



}
*/