#version 330

const float TWO_PI	= 6.2831853071795;
const float E			= 2.7182818284590;

uniform sampler2D colorTexture;


uniform vec2 pixelSize; // 1.0/width 1.0/height
uniform vec2 direction=vec2(1.0,0.0);

#define MAX_BLUR_RADIUS 4096

uniform int blurRadius = 16;
//uniform float uBlurSigma = 2.0f;

noperspective in vec2 fUVs;

out vec4 fragColor;



/*----------------------------------------------------------------------------*/
/*	Incremental, forward-differencing Gaussian elimination based on:
	http://http.developer.nvidia.com/GPUGems3/gpugems3_ch40.html */
vec4 incrementalGauss1D(
	in sampler2D srcTex, 
	in vec2 srcTexelSize, 
	in vec2 origin,
	in int radius,
	in vec2 direction
) {

	int nSamples = clamp(radius, 1, int(MAX_BLUR_RADIUS)) / 2;
	
	if (nSamples == 0)
		return texture(srcTex, origin);
	
	float SIGMA = float(blurRadius) / 8.0;
	float sig2 = SIGMA * SIGMA;
		
//	set up incremental counter:
	vec3 gaussInc;
	gaussInc.x = 1.0 / (sqrt(TWO_PI) * SIGMA);
	gaussInc.y = exp(-0.5 / sig2);
	gaussInc.z = gaussInc.y * gaussInc.y;
	
//	accumulate results:
	vec4 result = texture(srcTex, origin) * gaussInc.x;	
	for (int i = 1; i < nSamples; ++i) {
		gaussInc.xy *= gaussInc.yz;
		
		vec2 offset = float(i) * direction * srcTexelSize;
		result += texture(srcTex, origin - offset) * gaussInc.x;
		result += texture(srcTex, origin + offset) * gaussInc.x;
	}
	
	return result;
}

void main()
{
	fragColor = incrementalGauss1D(colorTexture, pixelSize, fUVs, blurRadius, direction); 
}