/*******************************************************************************
	Copyright (C) 2013 John Chapman

	This software is distributed freely under the terms of the MIT License.
	See "license.txt" or "http://copyfree.org/licenses/mit/license.txt".
*******************************************************************************/
/*
Adapted by Teo for MA48-Unlimited
*/

#version 330

uniform sampler2D colorTexture;
uniform sampler1D uLensColor;

const int MAX_SAMPLES = 64;

uniform int uSamples = 11; // number of ghost samples
uniform float uDispersal = 0.25; // dispersion factor

uniform float uHaloWidth = 0.6;
uniform float uDistortion = 3.0;

noperspective in vec2 fUVs;

out vec4 fragColor;

/*----------------------------------------------------------------------------*/
vec4 textureDistorted(
	in sampler2D tex, 
	in vec2 texcoord, 
	in vec2 direction,
	in vec3 distortion 
) {
	return vec4(
		texture(tex, texcoord + direction * distortion.r).r,
		texture(tex, texcoord + direction * distortion.g).g,
		texture(tex, texcoord + direction * distortion.b).b,
		1.0
	);
}

/*----------------------------------------------------------------------------*/
void main() {
	vec2 texcoord = -fUVs + vec2(1.0); // flip texcoordoords
	vec2 texelSize = 1.0 / vec2(textureSize(colorTexture, 0));
	
	// ghost vector to image centre:
   	vec2 ghostVec = (vec2(0.5) - texcoord) * uDispersal;
	vec2 haloVec = normalize(ghostVec) * uHaloWidth;
	
	vec3 distortion = vec3(-texelSize.x * uDistortion, 0.0, texelSize.x * uDistortion);

// sample ghosts:
	vec4 result = vec4(0.0);
	for (int i = 0; i < uSamples; ++i) {
		vec2 offset = fract(texcoord + ghostVec * float(i));
		
		float weight = length(vec2(0.5) - offset) / length(vec2(0.5));
		weight = pow(1.0 - weight, 10.0);
	
		result += textureDistorted(
			colorTexture,
			offset,
			normalize(ghostVec),
			distortion
		) * weight;
	}
	//give the ghosts a color
	result *= texture(uLensColor, length(vec2(0.5) - texcoord) / length(vec2(0.5)));

//	sample halo:
	float weight = length(vec2(0.5) - fract(texcoord + haloVec)) / length(vec2(0.5));
	weight = pow(1.0 - weight, 10.0);

	result += textureDistorted(
		colorTexture,
		fract(texcoord + haloVec),
		normalize(ghostVec),
		distortion
	) * weight;
	
	fragColor = result;
}