#version 330

uniform sampler2D colorTexture;
uniform int useTexture;
uniform int spotLightsOn=1;
uniform vec3 matColor;
uniform vec3 eyePos;
uniform float matShininess;
in vec3 worldNormal;
in vec2 fUVs;
in vec3 vertexPositionWorld;
out vec4 fragColor;

//Illumintaion Data
uniform vec3 ambientlight;

struct PointLight {
    vec3 lightColor;
    // Color of that point light
    vec3 position;
    float fConstantAtt;
    float fLinearAtt;
    float fExpAtt;
};

struct SpotLight {
    vec3 lightColor;
    // Color of that spot light
    vec3 position;
    vec3 direction;
    float fConeAngle, fConeCosine;
    float fLinearAtt;
};



uniform PointLight lights[1];
uniform SpotLight spotLights[2];

vec4 getPointLightColor(const PointLight ptLight)
{
    vec3 vPosToLight = lights[0].position - vertexPositionWorld;
    float fDist = length(vPosToLight);
    vPosToLight = normalize(vPosToLight);
    float fDiffuse = max(0.0, dot(vPosToLight,normalize(worldNormal)));
    float fAttTotal = ptLight.fConstantAtt + ptLight.fLinearAtt*fDist + ptLight.fExpAtt*fDist*fDist;
    return vec4(ptLight.lightColor, 1.0)*(fDiffuse)/fAttTotal;
    //return vec4(ptLight.lightColor, 1.0)*fDiffuse;
}

vec4 getSpotLightColor(const SpotLight spotLight) 
{
    if(spotLightsOn == 0)return vec4(0.0, 0.0, 0.0, 0.0);
    vec3 vDir = spotLight.position - vertexPositionWorld;
    float fDistance = length(vDir);
    vDir = normalize(vDir);
    float fCosine = dot(spotLight.direction, vDir);
    float fDif = 1.0-spotLight.fConeCosine;
    float fFactor = clamp((fCosine-spotLight.fConeCosine)/fDif, 0.0, 1.0);
    if(fCosine > spotLight.fConeCosine) 
		return vec4(spotLight.lightColor, 1.0)*fFactor/(fDistance*spotLight.fLinearAtt);
    return vec4(0.0, 0.0, 0.0, 0.0);
}



void main()
{
    //light calcs DIFFUSE
	//calcDirection
	
	vec3 lightVector = normalize(lights[0].position - vertexPositionWorld);
    /*
	vec4 diffuseLight= vec4(0, 0, 0, 0);
	vec3 lightVector = normalize(lightPos - vertexPositionWorld);
	float brightness = dot(lightVector,normalize(worldNormal));
	if(brightness > 0)
	{
		diffuseLight=vec4(brightness,brightness,brightness, 1.0f);
	}
	*/
	vec4 diffuseLight=getPointLightColor(lights[0]);
    //vec4 diffuseLight=vec4(1,0,0,0);
	
	//light calcs SPECULARdiffuseLight
	vec4 specularLight = vec4(0, 0, 0, 0);
    vec3 reflectedLightVectorWorld = normalize(reflect(-lightVector, worldNormal));
    vec3 eyeVecWorld = normalize(eyePos - vertexPositionWorld);
    float specularFactor = dot(reflectedLightVectorWorld,eyeVecWorld);
    specularFactor = pow(specularFactor,matShininess);
    if (specularFactor > 0) {
        specularLight = vec4(specularFactor, specularFactor, specularFactor, 1.0f);
    }


	
	vec4 intensity = diffuseLight + vec4(ambientlight,1.0f) + specularLight;
    for(int l=0;l<2;l++){
        intensity = intensity + getSpotLightColor(spotLights[l]);
    }


	if(useTexture==1)
	{
        fragColor = texture2D(colorTexture, fUVs.xy) * intensity;
    }

 else 
	{
        fragColor= vec4(matColor,1.0f) * intensity;
    }


} 
